﻿namespace core.TagInterface
{
    using core.IO;
    using System;
    using System.Collections.Generic;

    public class Flags : BaseObject
    {
        public int BitCount;
        public BitmaskTypeEnum BitmaskType;
        public List<Bit> Bits;
        public int Value;
        private bool verified;

        public Flags(string name, BitmaskTypeEnum type) : base(name)
        {
            this.Bits = null;
            this.verified = false;
            this.BitmaskType = type;
            switch (this.BitmaskType)
            {
                case BitmaskTypeEnum.Byte:
                    this.BitCount = 8;
                    base.size = 1;
                    break;

                case BitmaskTypeEnum.Word:
                    this.BitCount = 0x10;
                    base.size = 2;
                    break;

                case BitmaskTypeEnum.Long:
                    this.BitCount = 0x20;
                    base.size = 4;
                    break;
            }
            base.type = BaseObjectType.Flags;
        }

        public Flags(string name, BitmaskTypeEnum type, List<Bit> bits) : base(name)
        {
            this.Bits = null;
            this.verified = false;
            this.BitmaskType = type;
            this.Bits = bits;
            switch (this.BitmaskType)
            {
                case BitmaskTypeEnum.Byte:
                    this.BitCount = 8;
                    base.size = 1;
                    break;

                case BitmaskTypeEnum.Word:
                    this.BitCount = 0x10;
                    base.size = 2;
                    break;

                case BitmaskTypeEnum.Long:
                    this.BitCount = 0x20;
                    base.size = 4;
                    break;
            }
            base.type = BaseObjectType.Flags;
        }

        public override BaseObject Clone()
        {
            core.TagInterface.Flags flags = new core.TagInterface.Flags(base.name, this.BitmaskType);
            flags.Value = this.Value;
            flags.BitCount = this.BitCount;
            flags.Bits = new List<Bit>();
            if (this.Bits != null)
            {
                flags.Bits.AddRange(this.Bits);
            }
            flags.SetMetaOffset(base.MetaOffset);
            flags.SetOffset(base.Offset);
            flags.SetMapOffset(base.MapOffset);
            flags.Name = base.Name;
            flags.SetType(base.Type);
            return flags;
        }

        private bool IsBitSet2(int bitmask, int bitpositionvalue)
        {
            while (bitpositionvalue >= 1)
            {
                bitpositionvalue--;
                bitmask = bitmask >> 1;
            }
            return ((bitmask & 1) == 1);
        }

        public void Read(ref EndianIO.EndianReader BR)
        {
            switch (this.BitmaskType)
            {
                case BitmaskTypeEnum.Byte:
                    this.Value = BR.ReadByte();
                    break;

                case BitmaskTypeEnum.Word:
                    this.Value = BR.ReadInt16();
                    break;

                case BitmaskTypeEnum.Long:
                    this.Value = BR.ReadInt32();
                    break;
            }
            this.ReloadBits();
        }

        private void ReloadBits()
        {
            if (!(this.verified && (this.Bits != null)))
            {
                this.verify();
            }
            int bitmaskType = (int) this.BitmaskType;
            for (int i = 0; i < bitmaskType; i++)
            {
                Bit bit = this.Bits[i];
                bit.BitEnabled = this.IsBitSet2(this.Value, i);
                this.Bits[i] = bit;
            }
        }

        private int SetBits()
        {
            uint num = 1;
            uint num2 = 0;
            for (int i = 0; i < this.Bits.Count; i++)
            {
                if (this.Bits[i].BitEnabled)
                {
                    num2 += num;
                }
                num = num << 1;
            }
            return (int) num2;
        }

        private void verify()
        {
            int num3;
            Bit current;
            int bitmaskType = (int) this.BitmaskType;
            if (this.Bits != null)
            {
                this.Bits.Sort(new BitSorter());
                List<Bit> list = new List<Bit>();
                int num2 = 0;
                for (num3 = 0; num3 < bitmaskType; num3++)
                {
                    bool flag = false;
                    using (List<Bit>.Enumerator TagEnumerator = this.Bits.GetEnumerator())
                    {
                        while (TagEnumerator.MoveNext())
                        {
                            current = TagEnumerator.Current;
                            if (current.BitPosition == num3)
                            {
                                list.Add(current);
                                flag = true;
                                goto Label_009A;
                            }
                        }
                    }
                Label_009A:
                    if (!flag)
                    {
                        current = new Bit();
                        current.BitName = "Unknown " + num2.ToString();
                        num2++;
                        current.BitPosition = num3;
                        list.Add(current);
                    }
                }
                this.Bits = list;
            }
            else
            {
                this.Bits = new List<Bit>();
                for (num3 = 0; num3 < bitmaskType; num3++)
                {
                    current = new Bit();
                    current.BitName = "Unkown " + num3.ToString();
                    current.BitPosition = num3;
                    this.Bits.Add(current);
                }
            }
            this.verified = true;
        }

        public void Write(ref EndianIO.EndianWriter BW)
        {
            this.Value = this.SetBits();
            switch (this.BitmaskType)
            {
                case BitmaskTypeEnum.Byte:
                    BW.Write((byte) this.Value);
                    break;

                case BitmaskTypeEnum.Word:
                    BW.Write((short) this.Value);
                    break;

                case BitmaskTypeEnum.Long:
                    BW.Write(this.Value);
                    break;
            }
        }

        public class Bit
        {
            public bool BitEnabled;
            public string BitName;
            public int BitPosition;

            public Bit()
            {
            }

            public Bit(string Name, int Position)
            {
                this.BitName = Name;
                this.BitPosition = Position;
            }
        }

        public enum BitmaskTypeEnum
        {
            Byte = 8,
            Long = 0x20,
            Word = 0x10
        }

        public class BitSorter : IComparer<core.TagInterface.Flags.Bit>
        {
            public int Compare(core.TagInterface.Flags.Bit x, core.TagInterface.Flags.Bit y)
            {
                return x.BitPosition.CompareTo(y.BitPosition);
            }
        }
    }
}

